/*
    FreeRTOS V6.1.0 - Copyright (C) 2010 Real Time Engineers Ltd.

    ***************************************************************************
    *                                                                         *
    * If you are:                                                             *
    *                                                                         *
    *    + New to FreeRTOS,                                                   *
    *    + Wanting to learn FreeRTOS or multitasking in general quickly       *
    *    + Looking for basic training,                                        *
    *    + Wanting to improve your FreeRTOS skills and productivity           *
    *                                                                         *
    * then take a look at the FreeRTOS books - available as PDF or paperback  *
    *                                                                         *
    *        "Using the FreeRTOS Real Time Kernel - a Practical Guide"        *
    *                  http://www.FreeRTOS.org/Documentation                  *
    *                                                                         *
    * A pdf reference manual is also available.  Both are usually delivered   *
    * to your inbox within 20 minutes to two hours when purchased between 8am *
    * and 8pm GMT (although please allow up to 24 hours in case of            *
    * exceptional circumstances).  Thank you for your support!                *
    *                                                                         *
    ***************************************************************************

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/*
 * This is a very simple demo that demonstrates task and queue usages only.
 * Details of other FreeRTOS features (API functions, tracing features,
 * diagnostic hook functions, memory management, etc.) can be found on the
 * FreeRTOS web site (http://www.FreeRTOS.org) and in the FreeRTOS book.
 * Details of this demo (what it does, how it should behave, etc.) can be found
 * in the accompanying PDF application note.
 *
*/

/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "LPC17xx.h"
#include "uart.h"
#include "wifly.h"
#include "semphr.h"
#include "utils.h"
#include "hmi.h"
#include "reg_datos.h"
#include "mdbm.h"
#include "mdbe.h"
#include "wdt.h"

/* The rate at which data is sent to the queue, specified in milliseconds. */
#define mainQUEUE_SEND_FREQUENCY_MS			( 100 / portTICK_RATE_MS )

/* The number of items the queue can hold.  This is 1 as the receive task
will remove items as they are added, meaning the send task should always find
the queue empty. */
#define mainQUEUE_LENGTH					( 1 )

/* Time out del Watchdog en segundos (aproximadamente) */
#define WDT_TIMEOUT 20

/* The task functions. */
void vTaskMdbm( void *pvParameters );
void vTaskHMI( void *pvParameters );
void vTaskMdbe( void *pvParameters );
void vTaskWifly( void *pvParameters );
void vTaskWdt( void *pvParameters );

/* The queue used by both tasks. */
static xQueueHandle xQueue = NULL;

/* Guarda el tick de la ultima vez que llego un caracter al hmi serie */
portTickType tickSistema;

uint8_t wdMdbm = 0;
uint8_t wdMdbe = 0;
uint8_t wdHMI = 0;
uint8_t wdWifly = 0;

extern UART_activa;

/*-----------------------------------------------------------*/

int main(void)
{
	uint8_t la_ret = 0;
	uint8_t la_intentos = 3;

	/* Inicia WiFly */
	/* Testeado el reinicio y configuracion del modulo WiFly */
	la_ret = wifly_Init();
	if (la_ret)
	{
		do
		{
			la_ret = wifly_Reset ();

			if (la_ret)
			{
				la_ret = wifly_configTelnet();
			}

			la_intentos--;
		}while (la_intentos && (la_ret == 0));
	}

	/* En el caso que tengamos configurado el modulo Wifly y salvada la configuracion podemos realizar
	 * la configuracion de la UART, ya que podemos realizar la conexion Telnet con el mismo
	 */
	/*
	UARTInit(UART_WIFLY, 38400);*/

	UARTInit(UART_HMI, 38400);

	UART_activa = UART_HMI;

	UARTInit(UART_MDBM, 9600);

	UARTInit(UART_MDBE, 9600);

	reg_datos_inicia ();

	mdbm_inicia();

	mdbe_inicia();

    xTaskCreate(vTaskMdbm, "Mdbm", 240, NULL, 1,	NULL );

	xTaskCreate(vTaskHMI, "HMI", 240, NULL, 1, NULL );

	xTaskCreate(vTaskMdbe, "Mdbe", 240, NULL, 1,	NULL );

	xTaskCreate(vTaskWifly, "Wifly", 240, NULL, 1,	NULL );

	xTaskCreate(vTaskWdt, "Wdt", 240, NULL, 1,	NULL );

	/* Start the tasks running. */
	vTaskStartScheduler();

	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
}

void vTaskHMI( void *pvParameters )
{
unsigned char la_car = 0; /* caracter a tratar */
uint16_t la_size = 0;     /* tamaño recibido */
uint16_t la_i;            /* indice */
char     la_cadena[10];   /* cadena de recepcion */
char     la_byte[1];      /* caracter para eco */
unsigned la_leidos=0;     /* caracteres leidos */
uint8_t  la_rec = 0;      /* indicacion de recepcion */

	/* Imprime el menu principal */
	hmi_menu();

	/* As per most tasks, this task is implemented in an infinite loop. */
	while(1)
	{
		wdHMI = 1;
		la_size = UART3Scan (HMI_RX_BUFF, 10);
		la_i=0;
		/* Si no tiene el control de la interface lo toma y presenta el menu principal */
		if ((UART_activa != UART_HMI) && la_size)
		{
			/* Recoger el tick del sistema */
			tickSistema = xTaskGetTickCount();

			UART_activa = UART_HMI;
			hmi_menu();
			la_size = 0;
		}
		for (;la_size;la_size--)
		{
			tickSistema = xTaskGetTickCount();
			la_car = HMI_RX_BUFF[la_i];
			la_i++;

			if ( la_car == '\r')
			{
				la_cadena[la_leidos] = '\0';
				la_leidos = 0;
				la_rec = 1;
				break;
			}
			/* caracter de backspace */
			else if ( la_car == 127 )
			{
				if ( la_leidos )
				{
					la_byte[0] = la_car;
					UARTSend (UART_activa, (uint8_t *)la_byte, 1);
					la_leidos -= 1;
				}
			}
			else
			{
				la_cadena[la_leidos] = la_car;
				la_byte[0] = la_car;
				UARTSend (UART_activa, (uint8_t *)la_byte, 1);
				la_leidos += 1;
				if (la_leidos > 8)
				{
					hmi_opcionNoValida();
					hmi_menu();
					la_leidos = 0;
				}
			}
		}

		if (la_rec)
		{
			hmi_estadoMenu ((char *)la_cadena);
			la_rec =0;
		}
		/* Delay for a period. */
		vTaskDelay(100);
	}
}


void vTaskMdbm( void *pvParameters )
{
	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		wdMdbm = 1;
		mdbm_pro();
		/* Delay for a period. */
		vTaskDelay(50);
	}
}

void vTaskMdbe( void *pvParameters )
{
	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		wdMdbe = 1;
		mdbe_pro();
		/* Delay for a period. */
		vTaskDelay(50);
	}
}

void vTaskWifly( void *pvParameters )
{
unsigned char la_car = 0;    /* caracter a tratar */
uint16_t      la_size = 0;   /* tamaño recibido */
uint16_t      la_i;          /* indice */
char          la_cadena[10]; /* cadena de recepcion */
char          la_byte[1];    /* caracter para eco */
unsigned      la_leidos=0;   /* caracteres leidos */
uint8_t       la_rec = 0;    /* indicacion de recepcion */
portTickType  la_tick;       /* tick actual */

	/* As per most tasks, this task is implemented in an infinite loop. */
	while(1)
	{
		wdWifly = 1;
		la_size = UART0Scan (WIFLY_RX_BUFF, 10);
		la_i=0;
		if ((UART_activa != UART_WIFLY) && la_size)
		{
			/* Recoger el tick del sistema */
			la_tick = xTaskGetTickCount();

			if ((la_tick - tickSistema) > HMI_TIEMPO_ESPERA)
			{
				UART_activa = UART_WIFLY;
				hmi_menu();
				la_size = 0;
			}
		}

		for (;la_size && (UART_activa == UART_WIFLY);la_size--)
		{
			la_car = WIFLY_RX_BUFF[la_i];
			la_i++;

			if ( la_car == '\r')
			{
				la_cadena[la_leidos] = '\0';
				la_leidos = 0;
				la_rec = 1;
				break;
			}
			/* caracter de backspace */
			else if ( la_car == 127 )
			{
				if ( la_leidos )
				{
					la_byte[0] = la_car;
					UARTSend (UART_activa, (uint8_t *)la_byte, 1);
					la_leidos -= 1;
				}
			}
			else
			{
				la_cadena[la_leidos] = la_car;
				la_byte[0] = la_car;
				UARTSend (UART_activa, (uint8_t *)la_byte, 1);
				la_leidos += 1;
				if (la_leidos > 8)
				{
					hmi_opcionNoValida();
					hmi_menu();
					la_leidos = 0;
				}
			}
		}

		if (la_rec)
		{
			hmi_estadoMenu ((char *)la_cadena);
			la_rec =0;
		}
		/* Delay for a period. */
		vTaskDelay(100);
	}
}

void vTaskWdt( void *pvParameters )
{
	WDTInit( WDT_TIMEOUT ); /* Aproximadamente 20 segundos */

	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		if (wdMdbm && wdMdbe && wdHMI && wdWifly)
		{
			WDTFeed();
			wdMdbm = 0;
			wdMdbe = 0;
			wdHMI = 0;
			wdWifly = 0;
		}
		/* Delay for a period. */
		vTaskDelay(200);
	}
}


/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */

	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
