/*
 * mdbe.h
 *
 *  Created on: 23/05/2014
 *      Author: Alfredo Florencio Jimenez
 */

#ifndef MDBE_H_
#define MDBE_H_

#include "uart.h"

/* Codigo de excepciones */

#define MDBE_EXCEPTION_ILLEGAL_FUNCTION       0x01
#define MDBE_EXCEPTION_ILLEGAL_DATA_ADDRESS   0x02
#define MDBE_EXCEPTION_ILLEGAL_DATA_VALUE     0x03

/* Estados de la linea de comunicacion */

#define MDBE_LINEA_REPOSO     0 /* Esperando peticion de equipo master */
#define MDBE_LINEA_RESPUESTA  1 /* Envia respuesta del slave */
#define MDBE_LINEA_MANTEN     2 /* Estado de mantenimiento */

/* Numero maximo de datos */
#define MDBE_MAX_DATA       260
/* Tiempo máximo entre caracteres de recepcion */
#define MDBE_CAR_TIMEOUT    200

/* Si se va a ejecutar en una maquina INTEL (LITTLE_ENDIAN) se define como 1. Si lo va a hacer
   en una MOTOROLA (BIG_ENDIAN) como 0. */
#define MDBECNF_LITTLE_ENDIAN_INTEL      1

#if (MDBECNF_LITTLE_ENDIAN_INTEL)

/* Para maquinas INTEL. */
#define MDBE_MSB   1
#define MDBE_LSB   0

#else

/* Para maquinas MOTOROLA. */
#define MDBE_MSB   0
#define MDBE_LSB   1

#endif

typedef union
{
	unsigned short s;
	unsigned char  b[2];
} UWB_mdbe_t;

typedef union
{
	short s;
	unsigned char  b[2];
} WB_mdbe_t;

typedef struct
{
	int size;
	unsigned char data[MDBE_MAX_DATA];
}mdbedefs_dataUnit_t;


typedef struct mdbebd_poll_str
{
	unsigned char        esperando;         /* Estado del poll de comunicaciones del modbus slave */
	unsigned char        codeFunction;      /* Codigo de funcion que nos piden */
	unsigned short       regIni;            /* Registro de inicio pedido */
	unsigned short       numReg;            /* Numero de registro pedidos */
	unsigned char        dispositivo;       /* Direccion de nuestro dispositivo */
	unsigned short       longitudEsperada;  /* Longitud esperada del mensaje */
	mdbedefs_dataUnit_t  lpduTra;           /* Trama de transmision */
	mdbedefs_dataUnit_t  lpduRec;           /* Trama de recepcion */
	unsigned long        tiempoBase;        /* Tiempo base de la linea */
	unsigned short       nChar;             /* Contador de caracteres */
	unsigned long        tiempoCar;         /* Tiempo de recepcion de caracter */
	unsigned char        bufferAux[MDBE_MAX_DATA];
}mdbebd_poll_t;

/* Buffer de recepcion */
char MDBE_RX_BUFF[BUFSIZE];

/*****************************************************************************
** Function name:		mdbe_pro
**
** Descriptions:		Funcion para el procesamiento de la linea de
**                      comunicaciones del modbus slave
**
** parameters:			none
** Returned value:		none
**
*****************************************************************************/
void mdbe_pro (void);

/*****************************************************************************
** Function name:		mdbe_inicia
**
** Descriptions:		Funcion para el inicio de la linea de
**                      comunicaciones del modbus slave
**
** parameters:			none
** Returned value:		none
**
*****************************************************************************/
void mdbe_inicia (void);

extern mdbebd_poll_t mdbe_poll;

#endif /* MDBE_H_ */
