/*
 * mdbm.h
 *
 *  Created on: 24/04/2014
 *      Author: Alfredo Florencio Jimenez
 */

#ifndef MDBM_H_
#define MDBM_H_

#include "uart.h"

/* Codigo de funcion de mensajes */

#define MDBM_READ_SINGLE_COIL          0x01  /* Sigle Coil */
#define MDBM_READ_INPUT_STATUS         0x02  /* Input Status */
#define MDBM_READ_HOLDING_REGISTER     0x03  /* Holding Register */
#define MDBM_READ_INPUT_REGISTER       0x04  /* Input Register */
#define MDBM_FORCE_SINGLE_COIL         0x05  /* Force Single Coil */
#define MDBM_PRESET_SINGLE_REGISTER    0x06  /* Preset Single Register */

/* Codigo de excepciones */

#define MDBM_EXCEPTION_ILLEGAL_FUNCTION       0x01
#define MDBM_EXCEPTION_ILLEGAL_DATA_ADDRESS   0x02
#define MDBM_EXCEPTION_ILLEGAL_DATA_VALUE     0x03

/* Estados de la linea de comunicacion */

#define MDBM_LINEA_REPOSO     0 /* Reposo de la linea de comunicacion */
#define MDBM_LINEA_RESPUESTA  1 /* Esperando respuesta de equipo esclavo */
#define MDBM_LINEA_MANTEN     2 /* Estado de mantenimiento */

/* Numero maximo de datos */
#define MDBM_MAX_DATA       260
/* Tiempo máximo entre caracteres de recepcion */
#define MDBM_CAR_TIMEOUT    200
/* Tiempo máximo de recepcion de mensaje */
#define MDBM_LINK_TIMEOUT   2500
/* Numero maximo de peticiones del maestro que podemos configurar */
#define MDBM_MAX_PET        20

/* Si se va a ejecutar en una maquina INTEL (LITTLE_ENDIAN) se define como 1. Si lo va a hacer
   en una MOTOROLA (BIG_ENDIAN) como 0. */
#define MDBMCNF_LITTLE_ENDIAN_INTEL      1

#if (MDBMCNF_LITTLE_ENDIAN_INTEL)

/* Para maquinas INTEL. */
#define MDBM_MSB   1
#define MDBM_LSB   0

#else

/* Para maquinas MOTOROLA. */
#define MDBM_MSB   0
#define MDBM_LSB   1

#endif


/* Tipos de datos */

typedef union
{
	unsigned short s;
	unsigned char  b[2];
} UWB_mdbm_t;

typedef union
{
	short s;
	unsigned char  b[2];
} WB_mdbm_t;


typedef struct
{
	int size;
	unsigned char data[MDBM_MAX_DATA];
}mdbmdefs_dataUnit_t;

typedef struct
{
	unsigned char  address;    /* direccion del equipo modbus */
	unsigned char  funCode;    /* codigo de funcion de la peticion */
	unsigned short addIni;     /* direccion del primer registro por el que se interroga */
	unsigned short numReg;     /* numero de registros que se solicita */
	unsigned short pInst;      /* punto donde se instala */
	unsigned long  ciclo;      /* tiempo en milisegundos para enviar */
	portTickType   tiempoBase; /* tiempo base desde que fue llamada */
}mdbm_peticion_t;


typedef struct mdbmbd_poll_str
{
   unsigned char        esperando;         /* Estado del poll de comunicaciones del modbus master */
   unsigned char        hayComando;        /* Indicacion de comando pendiente de ejecutar */
   unsigned char        indicePeticion;    /* Peticion activa */
   unsigned char        codeFunction;      /* Codigo de funcion que esperamos */
   unsigned char        dispositivo;       /* Direccion del dispositivo del que esperamos respuesta */
   unsigned short       longitudEsperada;  /* Longitud esperada del mensaje */
   mdbmdefs_dataUnit_t  lpduRec;           /* Trama de recepcion */
   unsigned long        tiempoBase;        /* Tiempo base de la linea */
   unsigned short       nChar;             /* Contador de caracteres */
   unsigned long        tiempoCar;         /* Tiempo de recepcion de caracter */
   unsigned char        frameError;        /* Indicacion de excepcion en la trama */
   mdbm_peticion_t      peticion[MDBM_MAX_PET];
}mdbmbd_poll_t;

/* Buffer de recepcion */
char MDBM_RX_BUFF[BUFSIZE];

/* Definicion de funciones */


/*****************************************************************************
** Function name:		mdbm_pro
**
** Descriptions:		Funcion para el procesamiento de la linea de
**                      comunicaciones del modbus master
**
** parameters:			none
** Returned value:		none
**
*****************************************************************************/
void mdbm_pro (void);

/*****************************************************************************
** Function name:		mdbm_inicia
**
** Descriptions:		Funcion para el inicio de la linea de
**                      comunicaciones del modbus master
**
** parameters:			none
** Returned value:		none
**
*****************************************************************************/
void mdbm_inicia (void);

extern mdbmbd_poll_t mdbm_poll;

#endif /* MDBM_H_ */
