/*
 * reg_datos.h
 *
 *  Created on: 12/05/2014
 *      Author: Alfredo Florencio Jimenez
 */

#ifndef REG_DATOS_H_
#define REG_DATOS_H_

#define RD_MAX_CS 200  /* Numero de puntos para Coil Status */
#define RD_MAX_IS 200  /* Numero de puntos para Input Status */
#define RD_MAX_HR 200  /* Numero de puntos para Holding Register */
#define RD_MAX_IR 200  /* Numero de puntos para Input Registers */

/* Se genera el mapa de puntos */
unsigned char RD_CoilStatus[RD_MAX_CS];
unsigned char RD_InputStatus[RD_MAX_IS];
short RD_HoldingRegisters[RD_MAX_HR];
short RD_InputRegisters[RD_MAX_IR];

/*****************************************************************************
** Function name:		reg_datos_inicia
**
** Descriptions:		Inicializa a 0 los puntos del mapa
**
** Parameters:			Ninguno
** Returned value:		1: semaforo creado
**                      0: semaforo no creado
**
*****************************************************************************/
uint8_t reg_datos_inicia (void);

/*****************************************************************************
** Function name:		reg_datosEscribeCS
**
** Descriptions:		Escribe datos en los Coil Status
**
** Parameters:			ar_punto: punto donde se escribe
**                      ar_valor: valor del punto
** Returned value:		Ninguno
**
*****************************************************************************/
void reg_datosEscribeCS (uint16_t ar_punto, uint8_t ar_valor);

/*****************************************************************************
** Function name:		reg_datosEscribeIS
**
** Descriptions:		Escribe datos en los Coil Status
**
** Parameters:			ar_punto: punto donde se escribe
**                      ar_valor: valor del punto
** Returned value:		Ninguno
**
*****************************************************************************/
void reg_datosEscribeIS (uint16_t ar_punto, uint8_t ar_valor);

/*****************************************************************************
** Function name:		reg_datosEscribeHR
**
** Descriptions:		Escribe datos en los Holding Registers
**
** Parameters:			ar_punto: punto donde se escribe
**                      ar_valor: valor del punto
** Returned value:		Ninguno
**
*****************************************************************************/
void reg_datosEscribeHR (uint16_t ar_punto, short ar_valor);

/*****************************************************************************
** Function name:		reg_datosEscribeIR
**
** Descriptions:		Escribe datos en los Input Registers
**
** Parameters:			ar_punto: punto donde se escribe
**                      ar_valor: valor del punto
** Returned value:		Ninguno
**
*****************************************************************************/
void reg_datosEscribeIR (uint16_t ar_punto, short ar_valor);

/*****************************************************************************
** Function name:		reg_datosLeerCS
**
** Descriptions:		Lee datos en los Coil Status
**
** Parameters:			ar_punto: punto donde se lee
** Returned value:		la_valor: valor del punto
**
*****************************************************************************/
uint8_t reg_datosLeerCS (uint16_t ar_punto);

/*****************************************************************************
** Function name:		reg_datosLeerIS
**
** Descriptions:		Lee datos en los Input Status
**
** Parameters:			ar_punto: punto donde se lee
** Returned value:		la_valor: valor del punto
**
*****************************************************************************/
uint8_t reg_datosLeerIS (uint16_t ar_punto);

/*****************************************************************************
** Function name:		reg_datosLeerHR
**
** Descriptions:		Lee datos en los Holding Registers
**
** Parameters:			ar_punto: punto donde se lee
** Returned value:		la_valor: valor del punto
**
*****************************************************************************/
short reg_datosLeerHR (uint16_t ar_punto);

/*****************************************************************************
** Function name:		reg_datosLeerIR
**
** Descriptions:		Lee datos en los Input Registers
**
** Parameters:			ar_punto: punto donde se lee
** Returned value:		la_valor: valor del punto
**
*****************************************************************************/
short reg_datosLeerIR (uint16_t ar_punto);

#endif /* REG_DATOS_H_ */
