/*
 * wifly.h
 *
 *  Created on: 01/04/2014
 *      Author: Usuario
 */

#ifndef WIFLY_H_
#define WIFLY_H_

#define RST_WIFLY_PIN 22

#define BUF_LONG  512


#define BAUD_UART 38400 //9600


#define WF_CMD_MODE  "$$$"
#define WF_GET_IP    "get ip\r\n"
#define WF_GET_CH    "show net\r\n"
#define WF_GET_RSSSI "show rssi\r\n"
#define WF_GET_TIME  "show time\r\n"
#define WF_GET_PWD   "get wl\r\n"
#define WF_SET_PROTO "set ip proto 18\r\n"
#define WF_SET_HOST  "set ip host 0\r\n"
#define WF_SET_HTTP  "set ip remote 80\r\n"
#define WF_SET_COMR  "set com remote 0\r\n"
#define WF_SET_OF_1  "set option format 1\r\n"
#define WF_OPEN      "open\r\n"
#define WF_SET_GET   "set com remote GET$"

#define TM_RESET  4000
#define TM_PING   10000
#define TM_ACK    1000
#define TM_JOIN   5000
#define TM_OPEN   2000


/*****************************************************************************
** Function name:		wifly_Ping
**
** Descriptions:		Envia un ping a la direccion determinada
**
** parameters:			address: direccion a la que realizamos ping
** Returned value:		0: no hay ping
**                      1: hay ping
**
*****************************************************************************/
uint8_t wifly_Ping (char *address); /* Testeado */

/*****************************************************************************
** Function name:		wifly_IP
**
** Descriptions:		Envia un comando para conocer la IP del WiFly
**
** parameters:			None
** Returned value:		IP del modulo
**                      NULL : no encuentra IP
**
*****************************************************************************/
char* wifly_IP (void); /* Testeado */

/*****************************************************************************
** Function name:		wifly_Canal
**
** Descriptions:		Envia un comando para conocer el canal del WiFly
**
** parameters:			None
** Returned value:		NULL: canal no encontrado
**                      canal encontrado
**
*****************************************************************************/
char* wifly_Canal (void); /* Testeado */

/*****************************************************************************
** Function name:		wifly_RSSI
**
** Descriptions:		Envia un comando para conocer la rssi del WiFly
**
** parameters:			None
** Returned value:		ret : valor de rssi
**
*****************************************************************************/
char* wifly_RSSI (void); /* Testeado */

/*****************************************************************************
** Function name:		wifly_Tiempo
**
** Descriptions:		Envia un comando para conocer el tiempo que el
**                      WiFly esta encendido
**
** parameters:			None
** Returned value:		NULL: tiempo no encontrado
**                      Tiempo encontrado
**
*****************************************************************************/
char* wifly_Tiempo (void); /* Testeado */

/*****************************************************************************
** Function name:		wifly_ConsultaPotencia
**
** Descriptions:		Envia un comando para conocer la potencia del
**                      Wifly
**
** parameters:			None
** Returned value:		NULL: potencia no encontrada
**                      potencia encontrada
**
*****************************************************************************/
char* wifly_ConsultaPotencia (void); /* Testeado */

/*****************************************************************************
** Function name:		wifly_CambiaPotencia
**
** Descriptions:		Envia un comando para cambiar la potencia del Wifly
**
** parameters:			potencia : valor numerico de la potencia
** Returned value:		0: comando no aceptado
**                      1: comando aceptado
**
*****************************************************************************/
uint8_t wifly_CambiaPotencia (uint8_t potencia); /* Testeado */

/*****************************************************************************
** Function name:		wifly_EnviaGET
**
** Descriptions:		Envia un comando GET
**
** parameters:			command : comando GET a ejecutar
**                      host : host al que se envia el comando
** Returned value:		ret : respuesta al envio
**
*****************************************************************************/
char* wifly_EnviaGET (char *command, char *host);

/*****************************************************************************
** Function name:		wifly_Reset
**
** Descriptions:		Resetea el WiFly y entra en modo comando
**
** parameters:			None
** Returned value:		0: estado desconocido
**                      1: WiFly reseteado
**
*****************************************************************************/
uint8_t wifly_Reset (void); /* Testeado */


/*****************************************************************************
** Function name:		wifly_Init
**
** Descriptions:		Inicia el semaforo del driver del WiFly
**
** parameters:			None
** Returned value:		1: semaforo creado
**                      0: semaforo no creado
**
*****************************************************************************/
uint8_t wifly_Init (void); /* Testeado */

/*****************************************************************************
** Function name:		wifly_ConectaWEP
**
** Descriptions:		Envia la key WEP
**
** parameters:			None
** Returned value:		0: key WEP no aceptada
**                      1: key WEP aceptada
**
*****************************************************************************/
uint8_t wifly_ConectaWEP (char *key); /* Testeado */

/*****************************************************************************
** Function name:		wifly_ConectaWAP
**
** Descriptions:		Envia la frase WAP
**
** parameters:			None
** Returned value:		0: key WAP no aceptada
**                      1: key WAP aceptada
**
*****************************************************************************/
uint8_t wifly_ConectaWAP (char *phrase); /* Testeado */

/*****************************************************************************
** Function name:		wifly_ConectaSSID
**
** Descriptions:		Conecta a la red
**
** parameters:			None
** Returned value:		0: No conecta
**                      1: Conecta
**
*****************************************************************************/
uint8_t wifly_ConectaSSID (char *ssid); /* Testeado */

/*****************************************************************************
** Function name:		wifly_configHTTP
**
** Descriptions:		Prepara el wifly para conexiones HTTP
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void wifly_configHTTP ();

/*****************************************************************************
** Function name:		wifly_configTelnet
**
** Descriptions:		Prepara el wifly para conexiones Telnet
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
uint8_t wifly_configTelnet ();

#endif /* WIFLY_H_ */
