/*
 * hmi.c
 *
 *  Created on: 05/05/2014
 *      Author: Alfredo Florencio Jimenez
 */

#include "LPC17xx.h"
#include "hmi.h"
#include "string.h"
#include "utils.h"
#include "uart.h"
#include "FreeRTOS.h"
#include "task.h"
#include "mdbm.h"
#include "mdbe.h"
#include "stdlib.h"
#include "stdio.h"
#include "reg_datos.h"

static void hmi_menuVelocidadM(void);
static void hmi_menuMando(void);
static void hmi_addressCom (void);
static void hmi_noHayRegistrosMaster (void);
static void hmi_menuRegistrosMaster(void);
static void hmi_addressRegAddress (void);
static void hmi_valorMando (void);
static void hmi_muestraRegistro (void);
static void hmi_muestraOptionReg (void);
static void hmi_cambioMaster (void);
static void hmi_menuModificaPet (void);
static void hmi_menuNuevaPet (void);
static void hmi_menuMuestraOpc (void);
static void hmi_addressPet (void);
static void hmi_menuNumReg (void);
static void hmi_menuCiclo (void);
static void hmi_menuPunto (void);
static void hmi_mensajeNoPermitido (void);
static void hmi_registroDatos (void);
static void hmi_menuConsultaDatos (void);
static void hmi_puntoConsulta (void);
static void hmi_menuModificaDatos (void);
static void hmi_menuPuntoValor (void);
static void hmi_menuVelocidadS(void);
static void hmi_menuSlaveAddress (void);


static void hmi_menuOpcion (char *par_cadena);
static void hmi_opcionComando (char *par_cadena);
static void hmi_opcionVelocidadM (char *par_cadena);
static void hmi_opcionAddCom (char *par_cadena);
static void hmi_opcionRegAdd (char *par_cadena);
static void hmi_opcionValorMando (char *par_cadena);
static void hmi_opcionRegMaster (char *par_cadena);
static void hmi_opcionOptReg (char *par_cadena);
static void hmi_opcionAddress (char *par_cadena);
static void hmi_opcionRegAddress (char *par_cadena);
static void hmi_opcionNumeroReg (char *par_cadena);
static void hmi_opcionCiclo (char *par_cadena);
static void hmi_opcionPunto (char *par_cadena);
static void hmi_opcionConsultaDatos(char *par_cadena);
static void hmi_opcionPuntoMuestra(char *par_cadena);
static void hmi_opcionModificaDatos(char *par_cadena);
static void hmi_opcionPuntoModifica(char *par_cadena);
static void hmi_opcionPuntoValor(char *par_cadena);
static void hmi_opcionVelocidadS (char *par_cadena);
static void hmi_opcionSlaveAddress (char *par_cadena);

static unsigned char hmi_estado = 0;  /* Estado del menu <0> Menu principal */

/* Estructura de peticion de mando */
mdbm_peticion_t hmi_peticion_mando;
/* Estructura de peticion */
mdbm_peticion_t hmi_peticion_aux;
/* Indice de peticion activa */
unsigned char indicePeticion;
/* Activacion desactivacion de linea modbus master */
unsigned char cambiaMaster = 0;
/* Activacion desactivacion de linea modbus slave */
unsigned char cambiaSlave = 0;
/* Tipo de punto */
unsigned char tipoPunto = 0;
/* Numero de punto */
unsigned char numPunto = 0;
/* Muesta eventos */
unsigned char hmiEvent = 0;

/* UART activa */
uint32_t UART_activa = 3;

/*****************************************************************************
** Function name:		hmi_menu   <0>
*****************************************************************************/
void hmi_menu (void)
{
	/* Indicamos que estamos en el menu principal */
	hmi_estado = 0;

	/* Imprimimos por pantalla */
	UARTSend (UART_activa, (uint8_t *)HMI_M_PR, strlen(HMI_M_PR));
	/* opcion muestra menu */
	UARTSend (UART_activa, (uint8_t *)HMI_O_MM, strlen(HMI_O_MM));
	/* opcion consulta datos */
	UARTSend (UART_activa, (uint8_t *)HMI_O_CD, strlen(HMI_O_CD));
	/* opcion modifica datos */
	UARTSend (UART_activa, (uint8_t *)HMI_O_MD, strlen(HMI_O_MD));
	/* opcion envia comandos */
	UARTSend (UART_activa, (uint8_t *)HMI_O_EC, strlen(HMI_O_EC));
	/* opcion registros modbus master */
	UARTSend (UART_activa, (uint8_t *)HMI_O_RM, strlen(HMI_O_RM));
	/* opcion velocidad master */
	UARTSend (UART_activa, (uint8_t *)HMI_O_VC, strlen(HMI_O_VC));
	/* opcion velocidad slave */
	UARTSend (UART_activa, (uint8_t *)HMI_O_VS, strlen(HMI_O_VS));
	/* opcion activar desactivar master */
	UARTSend (UART_activa, (uint8_t *)HMI_O_AM, strlen(HMI_O_AM));
	/* opcion activar desactivar slave */
	UARTSend (UART_activa, (uint8_t *)HMI_O_AS, strlen(HMI_O_AS));
	/* opcion direccion de slave */
	UARTSend (UART_activa, (uint8_t *)HMI_O_SA, strlen(HMI_O_SA));
	/* opcion activar / desactivar escritura en registro de datos */
	UARTSend (UART_activa, (uint8_t *)HMI_O_HE, strlen(HMI_O_HE));
	/* menu opcion */
	UARTSend (UART_activa, (uint8_t *)HMI_M_OP, strlen(HMI_M_OP));
}

/*****************************************************************************
** Function name:		hmi_menuVelocidadM  <1>
*****************************************************************************/
static void hmi_menuVelocidadM(void)
{
	/* Indicamos que estamos en el menu cambio de velocidad */
	hmi_estado = 1;

	/* Imprimimos por pantalla */
	UARTSend (UART_activa, (uint8_t *)HMI_M_VM, strlen(HMI_M_VM));
	/* opcion 9600 */
	UARTSend (UART_activa, (uint8_t *)HMI_O_V1, strlen(HMI_O_V1));
	/* opcion 19200 */
	UARTSend (UART_activa, (uint8_t *)HMI_O_V2, strlen(HMI_O_V2));
	/* opcion 38400 */
	UARTSend (UART_activa, (uint8_t *)HMI_O_V3, strlen(HMI_O_V3));
	/* opcion salir menu */
	UARTSend (UART_activa, (uint8_t *)HMI_O_SM, strlen(HMI_O_SM));
	UARTSend (UART_activa, (uint8_t *)HMI_M_OP, strlen(HMI_M_OP));

}

/*****************************************************************************
** Function name:		hmi_menuMando  <2>
*****************************************************************************/
static void hmi_menuMando(void)
{
	/* Indicamos que estamos en el menu comando */
	hmi_estado = 2;

	/* Imprimimos por pantalla */
	UARTSend (UART_activa, (uint8_t *)HMI_M_EC, strlen(HMI_M_EC));
	/* opcion write coil */
	UARTSend (UART_activa, (uint8_t *)HMI_O_WC, strlen(HMI_O_WC));
	/* opcion write register */
	UARTSend (UART_activa, (uint8_t *)HMI_O_WR, strlen(HMI_O_WR));
	/* opcion salir menu */
	UARTSend (UART_activa, (uint8_t *)HMI_O_SM, strlen(HMI_O_SM));
	UARTSend (UART_activa, (uint8_t *)HMI_M_OP, strlen(HMI_M_OP));

}

/*****************************************************************************
** Function name:		hmi_opcionNoValida
*****************************************************************************/
void hmi_opcionNoValida (void)
{
	/* Imprimimos por pantalla */
	UARTSend (UART_activa, (uint8_t *)HMI_M_ON, strlen(HMI_M_ON));
	UARTSend (UART_activa, (uint8_t *)HMI_M_OP, strlen(HMI_M_OP));
}

/*****************************************************************************
** Function name:		hmi_addressCom  <3>
*****************************************************************************/
static void hmi_addressCom (void)
{
	/* Indicamos que estamos en el menu de direccion de equipo esclavo para mando*/
	hmi_estado = 3;

	/* opcion salir menu */
	UARTSend (UART_activa, (uint8_t *)HMI_O_SM, strlen(HMI_O_SM));
	UARTSend (UART_activa, (uint8_t *)HMI_M_CA, strlen(HMI_M_CA));

}

/*****************************************************************************
** Function name:		hmi_addressRegAddress  <4>
*****************************************************************************/
static void hmi_addressRegAddress (void)
{
	/* Indicamos que estamos en el menu de direccion de registro para equipo esclavo para mando*/
	hmi_estado = 4;

	UARTSend (UART_activa, (uint8_t *)HMI_M_RA, strlen(HMI_M_RA));

}

/*****************************************************************************
** Function name:		hmi_valorMando  <5>
*****************************************************************************/
static void hmi_valorMando (void)
{
	/* Indicamos que estamos en el menu de direccion de registro para equipo esclavo para mando*/
	hmi_estado = 5;

	UARTSend (UART_activa, (uint8_t *)HMI_M_CV, strlen(HMI_M_CV));

}

/*****************************************************************************
** Function name:		hmi_menuRegistrosMaster  <6>
*****************************************************************************/
static void hmi_menuRegistrosMaster (void)
{
	/* Indicamos que estamos en el menu de registros del Modbus master */
	hmi_estado = 6;

	/* Mustra en pantalla el registro actual */
	hmi_muestraRegistro ();

	/* Muestra el menu de seleccion de los registros del Modbus master */
	hmi_muestraOptionReg ();
}

/*****************************************************************************
** Function name:		hmi_muestraOptionReg
*****************************************************************************/
static void hmi_muestraOptionReg (void)
{
	/* opcion siguiente registro */
	UARTSend (UART_activa, (uint8_t *)HMI_O_SR, strlen(HMI_O_SR));
	/* opcion modificar registro */
	UARTSend (UART_activa, (uint8_t *)HMI_O_MR, strlen(HMI_O_MR));
	/* opcion borrar registro */
	UARTSend (UART_activa, (uint8_t *)HMI_O_BR, strlen(HMI_O_BR));
	/* opcion borrar todos los registros */
	UARTSend (UART_activa, (uint8_t *)HMI_O_BT, strlen(HMI_O_BT));
	/* opcion borrar todos los registros */
	UARTSend (UART_activa, (uint8_t *)HMI_O_NR, strlen(HMI_O_NR));
	/* opcion salir al menu principal */
	UARTSend (UART_activa, (uint8_t *)HMI_O_SM, strlen(HMI_O_SM));
	/* menu opcion */
	UARTSend (UART_activa, (uint8_t *)HMI_M_OP, strlen(HMI_M_OP));
}

/*****************************************************************************
** Function name:		hmi_noHayRegistrosMaster
*****************************************************************************/
static void hmi_noHayRegistrosMaster (void)
{
	/* Imprimimos por pantalla */
	UARTSend (UART_activa, (uint8_t *)HMI_M_NR, strlen(HMI_M_NR));
}

/*****************************************************************************
** Function name:		hmi_cambioMaster
*****************************************************************************/
static void hmi_cambioMaster (void)
{
	/* Imprimimos por pantalla */
	UARTSend (UART_activa, (uint8_t *)HMI_M_MM, strlen(HMI_M_MM));
}

/*****************************************************************************
** Function name:		hmi_muestraRegistro
*****************************************************************************/
static void hmi_muestraRegistro (void)
{
	char la_aux[64];
	/* Imprimimos por pantalla */

	/* direccion del equipo modbus */
	sprintf (la_aux,"\r\n\r\n Registro actual:\r\n     Direccion de slave: %d", mdbm_poll.peticion[indicePeticion].address);
	UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
	/* codigo de funcion de la peticion */
	sprintf (la_aux,"\r\n     Codigo de funcion: %d", mdbm_poll.peticion[indicePeticion].funCode);
	UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
	/* direccion del primer registro por el que se interroga */
	sprintf (la_aux,"\r\n     Registro de inicio: %d", mdbm_poll.peticion[indicePeticion].addIni);
	UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
	/* Numero de registros */
	sprintf (la_aux,"\r\n     Numero de registros: %d", mdbm_poll.peticion[indicePeticion].numReg);
	UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
	/* Punto donde se instala */
	sprintf (la_aux,"\r\n     Punto donde se instala: %d", mdbm_poll.peticion[indicePeticion].pInst);
	UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
	/* Tiempo de ciclo de preguntas */
	sprintf (la_aux,"\r\n     Ciclo en milisegundos: %d\r\n", mdbm_poll.peticion[indicePeticion].ciclo);
	UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
}

/*****************************************************************************
** Function name:		hmi_menuModificaPet  <7>
*****************************************************************************/
static void hmi_menuModificaPet (void)
{
	hmi_estado = 7;

	/* Imprimimos por pantalla */
	UARTSend (UART_activa, (uint8_t *)HMI_M_MP, strlen(HMI_M_MP));
	hmi_menuMuestraOpc();
}

/*****************************************************************************
** Function name:		hmi_menuNuevaPet  <8>
*****************************************************************************/
static void hmi_menuNuevaPet (void)
{
	hmi_estado = 8;

	/* Imprimimos por pantalla */
	UARTSend (UART_activa, (uint8_t *)HMI_M_NP, strlen(HMI_M_NP));
	hmi_menuMuestraOpc();
}

/*****************************************************************************
** Function name:		hmi_menuMuestraOpc
*****************************************************************************/
static void hmi_menuMuestraOpc (void)
{
	/* opcion coil status*/
	UARTSend (UART_activa, (uint8_t *)HMI_O_CS, strlen(HMI_O_CS));
	/* opcion input status */
	UARTSend (UART_activa, (uint8_t *)HMI_O_IS, strlen(HMI_O_IS));
	/* opcion holding register */
	UARTSend (UART_activa, (uint8_t *)HMI_O_HR, strlen(HMI_O_HR));
	/* opcion input register */
	UARTSend (UART_activa, (uint8_t *)HMI_O_IR, strlen(HMI_O_IR));
	/* opcion write coil */
	UARTSend (UART_activa, (uint8_t *)HMI_O_WC, strlen(HMI_O_WC));
	/* opcion write register */
	UARTSend (UART_activa, (uint8_t *)HMI_O_WR, strlen(HMI_O_WR));
	/* opcion salir al menu principal */
	UARTSend (UART_activa, (uint8_t *)HMI_O_SM, strlen(HMI_O_SM));
	/* menu opcion */
	UARTSend (UART_activa, (uint8_t *)HMI_M_OP, strlen(HMI_M_OP));
}

/*****************************************************************************
** Function name:		hmi_addressPet  <9>  <10>
*****************************************************************************/
static void hmi_addressPet (void)
{
	/* Indicamos que estamos en el menu de direccion de equipo esclavo para peticion*/
	if ((hmi_estado == 7) || (hmi_estado == 8))
	{
		hmi_estado = hmi_estado + 2;
	}

	/* opcion salir menu */
	UARTSend (UART_activa, (uint8_t *)HMI_O_SM, strlen(HMI_O_SM));
	UARTSend (UART_activa, (uint8_t *)HMI_M_CA, strlen(HMI_M_CA));

}

/*****************************************************************************
** Function name:		hmi_menuRegAddress  <11>  <12>
*****************************************************************************/
static void hmi_menuRegAddress (void)
{
	/* Indicamos que estamos en el menu de direccion de registro para la peticion */
	hmi_estado = hmi_estado + 2;

	UARTSend (UART_activa, (uint8_t *)HMI_M_RA, strlen(HMI_M_RA));

}

/*****************************************************************************
** Function name:		hmi_menuNumReg  <13>  <14>
*****************************************************************************/
static void hmi_menuNumReg (void)
{
	/* Indicamos que estamos en el menu de direccion de registro para la peticion */
	hmi_estado = hmi_estado + 2;

	if (hmi_peticion_aux.funCode > 4)
	{
		UARTSend (UART_activa, (uint8_t *)HMI_M_CV, strlen(HMI_M_CV));
	}
	else
	{
		UARTSend (UART_activa, (uint8_t *)HMI_M_RV, strlen(HMI_M_RV));
	}
}

/*****************************************************************************
** Function name:		hmi_menuCiclo  <15>  <16>
*****************************************************************************/
static void hmi_menuCiclo (void)
{
	/* Indicamos que estamos en el menu de direccion de registro para la peticion */
	hmi_estado = hmi_estado + 2;

	UARTSend (UART_activa, (uint8_t *)HMI_M_TC, strlen(HMI_M_TC));
}


/*****************************************************************************
** Function name:		hmi_menuPunto  <17>  <18>
*****************************************************************************/
static void hmi_menuPunto (void)
{
	/* Indicamos que estamos en el menu de direccion de registro para la peticion */
	hmi_estado = hmi_estado + 2;

	UARTSend (UART_activa, (uint8_t *)HMI_M_PI, strlen(HMI_M_PI));
}

/*****************************************************************************
** Function name:		hmi_mensajeNoPermitido
*****************************************************************************/
static void hmi_mensajeNoPermitido (void)
{
	UARTSend (UART_activa, (uint8_t *)HMI_M_MN, strlen(HMI_M_MN));
}

/*****************************************************************************
** Function name:		hmi_registroDatos
*****************************************************************************/
static void hmi_registroDatos (void)
{
	UARTSend (UART_activa, (uint8_t *)HMI_M_RS, strlen(HMI_M_RS));
}


/*****************************************************************************
** Function name:		hmi_mensajeNoEspacio
*****************************************************************************/
static void hmi_mensajeNoEspacio (void)
{
	UARTSend (UART_activa, (uint8_t *)HMI_M_NE, strlen(HMI_M_NE));
}

/*****************************************************************************
** Function name:		hmi_menuConsultaDatos  <19>
*****************************************************************************/
static void hmi_menuConsultaDatos (void)
{
	hmi_estado = 19;

	/* Menu de consulta de datos */
	UARTSend (UART_activa, (uint8_t *)HMI_M_CD, strlen(HMI_M_CD));
	/* opcion coil status */
	UARTSend (UART_activa, (uint8_t *)HMI_O_CS, strlen(HMI_O_CS));
	/* opcion input status */
	UARTSend (UART_activa, (uint8_t *)HMI_O_IS, strlen(HMI_O_IS));
	/* opcion holding register */
	UARTSend (UART_activa, (uint8_t *)HMI_O_HR, strlen(HMI_O_HR));
	/* opcion input register */
	UARTSend (UART_activa, (uint8_t *)HMI_O_IR, strlen(HMI_O_IR));
	/* opcion salir al menu principal */
	UARTSend (UART_activa, (uint8_t *)HMI_O_SM, strlen(HMI_O_SM));
	/* menu opcion */
	UARTSend (UART_activa, (uint8_t *)HMI_M_OP, strlen(HMI_M_OP));
}

/*****************************************************************************
** Function name:		hmi_puntoConsulta  <20>
*****************************************************************************/
static void hmi_puntoConsulta (void)
{
	hmi_estado = 20;
	/* Punto a mostrar */
	UARTSend (UART_activa, (uint8_t *)HMI_M_PM, strlen(HMI_M_PM));
}

/*****************************************************************************
** Function name:		hmi_menuModificaDatos  <21>
*****************************************************************************/
static void hmi_menuModificaDatos (void)
{
	hmi_estado = 21;

	/* Menu de consulta de datos */
	UARTSend (UART_activa, (uint8_t *)HMI_M_MD, strlen(HMI_M_MD));
	/* opcion coil status */
	UARTSend (UART_activa, (uint8_t *)HMI_O_CS, strlen(HMI_O_CS));
	/* opcion input status */
	UARTSend (UART_activa, (uint8_t *)HMI_O_IS, strlen(HMI_O_IS));
	/* opcion holding register */
	UARTSend (UART_activa, (uint8_t *)HMI_O_HR, strlen(HMI_O_HR));
	/* opcion input register */
	UARTSend (UART_activa, (uint8_t *)HMI_O_IR, strlen(HMI_O_IR));
	/* opcion salir al menu principal */
	UARTSend (UART_activa, (uint8_t *)HMI_O_SM, strlen(HMI_O_SM));
	/* menu opcion */
	UARTSend (UART_activa, (uint8_t *)HMI_M_OP, strlen(HMI_M_OP));
}

/*****************************************************************************
** Function name:		hmi_puntoModifica  <22>
*****************************************************************************/
static void hmi_puntoModifica (void)
{
	hmi_estado = 22;
	/* Punto a mostrar */
	UARTSend (UART_activa, (uint8_t *)HMI_M_PN, strlen(HMI_M_PN));
}

/*****************************************************************************
** Function name:		hmi_menuPuntoValor  <23>
*****************************************************************************/
static void hmi_menuPuntoValor (void)
{
	hmi_estado = 23;
	/* Mensaje para valor de status */
	UARTSend (UART_activa, (uint8_t *)HMI_M_MV, strlen(HMI_M_MV));
	/* Valor del punto */
	UARTSend (UART_activa, (uint8_t *)HMI_M_PV, strlen(HMI_M_PV));
}

/*****************************************************************************
** Function name:		hmi_menuVelocidadS  <24>
*****************************************************************************/
static void hmi_menuVelocidadS(void)
{
	/* Indicamos que estamos en el menu cambio de velocidad */
	hmi_estado = 24;

	/* Imprimimos por pantalla */
	UARTSend (UART_activa, (uint8_t *)HMI_M_VS, strlen(HMI_M_VS));
	/* opcion 9600 */
	UARTSend (UART_activa, (uint8_t *)HMI_O_V1, strlen(HMI_O_V1));
	/* opcion 19200 */
	UARTSend (UART_activa, (uint8_t *)HMI_O_V2, strlen(HMI_O_V2));
	/* opcion 38400 */
	UARTSend (UART_activa, (uint8_t *)HMI_O_V3, strlen(HMI_O_V3));
	/* opcion salir menu */
	UARTSend (UART_activa, (uint8_t *)HMI_O_SM, strlen(HMI_O_SM));
	UARTSend (UART_activa, (uint8_t *)HMI_M_OP, strlen(HMI_M_OP));

}


/*****************************************************************************
** Function name:		hmi_menuSlaveAddress  <25>
*****************************************************************************/
static void hmi_menuSlaveAddress (void)
{
	/* Indicamos que estamos en el menu de direccion de equipo esclavo para mando*/
	hmi_estado = 25;

	/* opcion salir menu */
	UARTSend (UART_activa, (uint8_t *)HMI_O_SM, strlen(HMI_O_SM));
	UARTSend (UART_activa, (uint8_t *)HMI_M_CA, strlen(HMI_M_CA));

}

/*****************************************************************************
** Function name:		hmi_estadoMenu
*****************************************************************************/
void hmi_estadoMenu (char *par_cadena)
{
	/* Vemos en que menu estamos */
	switch (hmi_estado)
	{
		case 0:
			hmi_menuOpcion(par_cadena);
		break;

		case 1:
			hmi_opcionVelocidadM(par_cadena);
		break;

		case 2:
			hmi_opcionComando(par_cadena);
		break;

		case 3:
			hmi_opcionAddCom(par_cadena);
		break;

		case 4:
			hmi_opcionRegAdd(par_cadena);
		break;

		case 5:
			hmi_opcionValorMando(par_cadena);
		break;

		case 6:
			hmi_opcionRegMaster(par_cadena);
		break;

		case 7:
		case 8:
			hmi_opcionOptReg(par_cadena);
		break;

		case 9:
		case 10:
			hmi_opcionAddress(par_cadena);
		break;

		case 11:
		case 12:
			hmi_opcionRegAddress(par_cadena);
		break;

		case 13:
		case 14:
			hmi_opcionNumeroReg(par_cadena);
		break;

		case 15:
		case 16:
			hmi_opcionCiclo(par_cadena);
		break;

		case 17:
		case 18:
			hmi_opcionPunto(par_cadena);
		break;

		case 19:
			hmi_opcionConsultaDatos(par_cadena);
		break;

		case 20:
			hmi_opcionPuntoMuestra(par_cadena);
		break;

		case 21:
			hmi_opcionModificaDatos(par_cadena);
		break;

		case 22:
			hmi_opcionPuntoModifica(par_cadena);
		break;

		case 23:
			hmi_opcionPuntoValor(par_cadena);
		break;

		case 24:
			hmi_opcionVelocidadS(par_cadena);
		break;

		case 25:
			hmi_opcionSlaveAddress(par_cadena);
		break;
		/* Estado desconocido volvemos al menu principal */
		default:
			hmi_menu();
	}
}

/*****************************************************************************
** Function name:		hmi_menuOpcion
*****************************************************************************/
static void hmi_menuOpcion (char *par_cadena)
{
	unsigned char la_car;        /* Primer caracter de la cadena */
	unsigned char la_i;          /* Indice de recorrido */
	unsigned char la_encontrado; /* Indicacion de para encontrar dato */

	la_car = *par_cadena;
	la_encontrado = 0;
	switch (la_car)
	{
		/* Muestra menu */
		case 'm':
		case 'M':
			hmi_menu();
		break;

		/* Activar / desactivar master */
		case 'a':
		case 'A':
			cambiaMaster = 1;
			if (mdbm_poll.esperando < MDBM_LINEA_MANTEN)
			{
				UARTSend (UART_activa, (uint8_t *)HMI_M_MO, strlen(HMI_M_MO));
			}
			else
			{
				for (la_i = 0; la_i < MDBM_MAX_PET; la_i++)
				{
					if (mdbm_poll.peticion[la_i].address)
					{
						la_encontrado = 1;
					}
				}
				if (la_encontrado)
				{
					UARTSend (UART_activa, (uint8_t *)HMI_M_ML, strlen(HMI_M_ML));
				}
				else
				{
					hmi_noHayRegistrosMaster();
				}
			}
			hmi_menu();
		break;

		/* Activar / desactivar master */
		case 's':
		case 'S':
			cambiaSlave = 1;
			if (mdbe_poll.esperando < MDBE_LINEA_MANTEN)
			{
				UARTSend (UART_activa, (uint8_t *)HMI_M_SO, strlen(HMI_M_SO));
			}
			else
			{
				if (mdbe_poll.dispositivo)
				{
					UARTSend (UART_activa, (uint8_t *)HMI_M_SL, strlen(HMI_M_SL));
				}
				else
				{
					UARTSend (UART_activa, (uint8_t *)HMI_M_SD, strlen(HMI_M_SD));
				}
			}
			hmi_menu();
		break;

		/* Consulta datos */
		case 'd':
		case 'D':
			hmi_menuConsultaDatos();
		break;

		/* Modifica datos */
		case 'z':
		case 'Z':
			hmi_menuModificaDatos();
		break;

		/* Envia comandos */
		case 'c':
		case 'C':
			hmi_menuMando();
		break;

		/* Registros master */
		case 'r':
		case 'R':
			/* Se busca el primer registro de peticiones del maestro */
			for (la_i = 0; ((la_i < MDBM_MAX_PET) && (!la_encontrado)); la_i++)
			{
				if (mdbm_poll.peticion[la_i].address)
				{
					la_encontrado = 1;
					/* Indicamos el indice de la primera peticion encontrada */
					indicePeticion = la_i;
				}
			}

			if (la_encontrado)
			{
				/* Se debe poner al Modbus master en estado de mantenimiento */
				hmi_menuRegistrosMaster ();
			}
			else
			{
				/* Indica que no existen registros */
				hmi_noHayRegistrosMaster();
				/* Vamos al menu de nueva peticion */
				hmi_menuNuevaPet();
			}
		break;

		/* Velocidad de comunicaciones master */
		case 'v':
		case 'V':
			hmi_menuVelocidadM();
		break;

		/* Velocidad de comunicaciones slave */
		case 'b':
		case 'B':
			hmi_menuVelocidadS();
		break;

		/* Direccion de comunicaciones slave */
		case 'k':
		case 'K':
			hmi_menuSlaveAddress();
		break;

		/* Habilita / deshabilita mostrar escritura en registro de datos */
		case 'e':
		case 'E':
			if (hmiEvent)
			{
				hmiEvent = 0;
			}
			else
			{
				hmiEvent = 1;
			}
		break;

		default:
			hmi_opcionNoValida();
    }
}

/*****************************************************************************
** Function name:		hmi_opcionComando
*****************************************************************************/
static void hmi_opcionComando (char *par_cadena)
{
	unsigned char la_car;

	la_car = *par_cadena;
	switch (la_car)
	{
		/* Muestra menu */
		case '0':
			hmi_menu();
		break;

		/* Comando a coil */
		case '5':
			hmi_peticion_mando.funCode = 5;
			/* peticion de direccion de esclavo */
			hmi_addressCom();
		break;

		/* Comando a register */
		case '6':
			hmi_peticion_mando.funCode = 6;
			/* peticion de direccion de esclavo */
			hmi_addressCom();
		break;

		default:
			hmi_opcionNoValida();
    }
}

/*****************************************************************************
** Function name:		hmi_opcionVelocidadM
*****************************************************************************/
static void hmi_opcionVelocidadM (char *par_cadena)
{
	unsigned char la_car;

	la_car = *par_cadena;
	switch (la_car)
	{
		/* Muestra menu */
		case '0':
			hmi_menu();
		break;

		/* Velocidad 9600 */
		case '1':
			UARTInit(UART_MDBM, 9600);
			hmi_menu();
		break;

		/* Velocidad 19200 */
		case '2':
			UARTInit(UART_MDBM, 19200);
			hmi_menu();
		break;

		/* Velocidad 38400 */
		case '3':
			UARTInit(UART_MDBM, 38400);
			hmi_menu();
		break;

		default:
			hmi_opcionNoValida();
    }
}

/*****************************************************************************
** Function name:		hmi_opcionAddCom
*****************************************************************************/
static void hmi_opcionAddCom (char *par_cadena)
{
	unsigned char la_address;

	la_address = atoi(par_cadena);

	if (la_address == 0)
	{
		hmi_menu();
	}
	else if ((la_address > 0) && (la_address < 248))
	{
		hmi_peticion_mando.address = la_address;
		hmi_addressRegAddress();
	}
	else
	{
		hmi_addressCom();
	}
}

/*****************************************************************************
** Function name:		hmi_opcionRegAdd
*****************************************************************************/
static void hmi_opcionRegAdd (char *par_cadena)
{
	unsigned short la_addReg;

	la_addReg = atoi(par_cadena);

	hmi_peticion_mando.addIni = la_addReg;

	hmi_valorMando();
}

/*****************************************************************************
** Function name:		hmi_opcionValorMando
*****************************************************************************/
static void hmi_opcionValorMando (char *par_cadena)
{
	unsigned short la_valMando;

	la_valMando = atoi(par_cadena);

	/* Para los mandos utilizamos el numReg para indicar el valor */

	/* Si el mando es un Coil cualquier valor distinto de 0 sera de activacion */
	if ((hmi_peticion_mando.funCode == 5) && (la_valMando))
	{
		hmi_peticion_mando.numReg = 0xFF00;
	}
	else
	{
		hmi_peticion_mando.numReg = la_valMando;
	}

	/* Activamos la señal de mando en el maestro y volvemos al menu principal */
	mdbm_poll.hayComando = 1;
	hmi_menu();
}

/*****************************************************************************
** Function name:		hmi_opcionRegMaster
*****************************************************************************/
static void hmi_opcionRegMaster (char *par_cadena)
{
	unsigned char la_car;
	unsigned char la_i;

	la_car = *par_cadena;
	/* ponemos el indice en la peticion actual */
	la_i = indicePeticion;

	switch (la_car)
	{
		/* Muestra menu */
		case '0':
			hmi_menu();
		break;

		/* Siguiente registro */
		case 'S':
		case 's':
			/* Se realiza un polling de las peticiones */
			do
			{
				la_i++;

				if (la_i >= MDBM_MAX_PET)
				{
					la_i = 0;
				}
			}while((la_i != indicePeticion) && (mdbm_poll.peticion[la_i].address == 0));
			indicePeticion = la_i;

			hmi_menuRegistrosMaster();
		break;

		/* Modificar registro */
		case 'M':
		case 'm':
			if (mdbm_poll.esperando == MDBM_LINEA_MANTEN)
			{
				hmi_menuModificaPet();

			}
			else
			{
				/* Mensaje para poner el master en mantenimiento */
				hmi_cambioMaster ();
				hmi_menu();
			}
		break;

		/* Nuevo registro */
		case 'N':
		case 'n':
			hmi_menuNuevaPet ();
		break;

		/* Borrar registro */
		case 'B':
		case 'b':
			if (mdbm_poll.esperando == MDBM_LINEA_MANTEN)
			{
				mdbm_poll.peticion[indicePeticion].address = 0;
			}
			else
			{
				/* Mensaje para poner el master en mantenimiento */
				hmi_cambioMaster ();
			}
			hmi_menu();
		break;

		/* Borrar todos los registros */
		case 'X':
		case 'x':
			if (mdbm_poll.esperando == MDBM_LINEA_MANTEN)
			{
				for (la_i = 0; la_i < MDBM_MAX_PET; la_i++)
				{
					mdbm_poll.peticion[la_i].address = 0;
				}
			}
			else
			{
				/* Mensaje para poner el master en mantenimiento */
				hmi_cambioMaster ();
			}
			hmi_menu();
		break;


		default:
			hmi_opcionNoValida();
    }
}

/*****************************************************************************
** Function name:		hmi_opcionOptReg
*****************************************************************************/
static void hmi_opcionOptReg (char *par_cadena)
{
	unsigned char la_car;

	la_car = *par_cadena;
	switch (la_car)
	{
		/* Muestra menu */
		case '0':
			hmi_menu();
		break;

		/* Coil status */
		case '1':
			hmi_peticion_aux.funCode = 1;
			/* peticion de direccion de esclavo */
			hmi_addressPet();
		break;

		/* Input status */
		case '2':
			hmi_peticion_aux.funCode = 2;
			/* peticion de direccion de esclavo */
			hmi_addressPet();
		break;

		/* Holding register */
		case '3':
			hmi_peticion_aux.funCode = 3;
			/* peticion de direccion de esclavo */
			hmi_addressPet();
		break;

		/* Input register */
		case '4':
			hmi_peticion_aux.funCode = 4;
			/* peticion de direccion de esclavo */
			hmi_addressPet();
		break;

		/* Comando a coil */
		case '5':
			hmi_peticion_aux.funCode = 5;
			/* peticion de direccion de esclavo */
			hmi_addressPet();
		break;

		/* Comando a register */
		case '6':
			hmi_peticion_aux.funCode = 6;
			/* peticion de direccion de esclavo */
			hmi_addressPet();
		break;

		default:
			hmi_opcionNoValida();
    }
}

/*****************************************************************************
** Function name:		hmi_opcionAddress
*****************************************************************************/
static void hmi_opcionAddress (char *par_cadena)
{
	unsigned char la_address;

	la_address = atoi(par_cadena);

	if (la_address == 0)
	{
		hmi_menu();
	}
	else if ((la_address > 0) && (la_address < 248))
	{
		hmi_peticion_aux.address = la_address;
		hmi_menuRegAddress();
	}
	else
	{
		hmi_addressPet();
	}
}

/*****************************************************************************
** Function name:		hmi_peticionRegAddress
*****************************************************************************/
static void hmi_opcionRegAddress (char *par_cadena)
{
	unsigned short la_addReg;

	la_addReg = atoi(par_cadena);

	hmi_peticion_aux.addIni = la_addReg;

	hmi_menuNumReg();
}

/*****************************************************************************
** Function name:		hmi_opcionNumeroReg
*****************************************************************************/
static void hmi_opcionNumeroReg (char *par_cadena)
{
	unsigned short la_numReg;

	la_numReg = atoi(par_cadena);

	/* Si el mando es un Coil cualquier valor distinto de 0 sera de activacion */
	if ((hmi_peticion_aux.funCode == 5) && (la_numReg))
	{
		hmi_peticion_aux.numReg = 0xFF00;
	}
	else
	{
		hmi_peticion_aux.numReg = la_numReg;
	}

	hmi_menuCiclo ();
}

/*****************************************************************************
** Function name:		hmi_opcionCiclo
*****************************************************************************/
static void hmi_opcionCiclo (char *par_cadena)
{
	unsigned short la_ciclo;

	la_ciclo = atoi(par_cadena);

	hmi_peticion_aux.ciclo = la_ciclo;

	hmi_menuPunto();
}

/*****************************************************************************
** Function name:		hmi_opcionPunto
*****************************************************************************/
static void hmi_opcionPunto (char *par_cadena)
{
	unsigned short la_punto;
	uint32_t la_aux;
	uint32_t la_max;
	unsigned char la_i;
	unsigned char la_encontrado = 0;

	la_punto = atoi(par_cadena);

	hmi_peticion_aux.pInst = la_punto;

	/* numero de regisros que se instalan mas el punto de instalacion */
	la_aux = hmi_peticion_aux.numReg + hmi_peticion_aux.pInst;

	switch (hmi_peticion_aux.funCode)
	{
		case 1:
			la_max = RD_MAX_CS;
		break;

		case 2:
			la_max = RD_MAX_IS;
		break;

		case 3:
			la_max = RD_MAX_HR;
		break;

		case 4:
			la_max = RD_MAX_IR;
		break;

		default:
			la_max = 0;
	}

	/* Comprobar que la peticion es correcta */
	/* Comprueba que no se piden mas registros de los permitidos */
	if (
			((hmi_peticion_aux.funCode < 3) && (hmi_peticion_aux.numReg > 2000)) ||
			((hmi_peticion_aux.funCode > 2) && (hmi_peticion_aux.funCode < 5) && (hmi_peticion_aux.numReg > 125))
		)
	{
		/* Mensaje de mas registros de los permitidos en la peticion */
		hmi_mensajeNoPermitido ();
	}
	else if ( (hmi_peticion_aux.funCode < 5) && (la_aux > la_max) )
	{
		/* Mensaje de instalacion de puntos fuera de rango */
		hmi_registroDatos();
	}
	else
	{
		/* Modificacion de registro */
		if (hmi_estado == 17)
		{
			mdbm_poll.peticion[indicePeticion].addIni = hmi_peticion_aux.addIni;
			mdbm_poll.peticion[indicePeticion].address = hmi_peticion_aux.address;
			mdbm_poll.peticion[indicePeticion].ciclo = hmi_peticion_aux.ciclo;
			mdbm_poll.peticion[indicePeticion].funCode = hmi_peticion_aux.funCode;
			mdbm_poll.peticion[indicePeticion].numReg = hmi_peticion_aux.numReg;
			mdbm_poll.peticion[indicePeticion].pInst = hmi_peticion_aux.pInst;
			mdbm_poll.peticion[indicePeticion].tiempoBase = 0;
		}
		if (hmi_estado == 18)
		{
			for (la_i = 0; ((la_i < MDBM_MAX_PET) && (!la_encontrado)); la_i++)
			{
				if (mdbm_poll.peticion[la_i].address == 0)
				{
					mdbm_poll.peticion[la_i].addIni = hmi_peticion_aux.addIni;
					mdbm_poll.peticion[la_i].address = hmi_peticion_aux.address;
					mdbm_poll.peticion[la_i].ciclo = hmi_peticion_aux.ciclo;
					mdbm_poll.peticion[la_i].funCode = hmi_peticion_aux.funCode;
					mdbm_poll.peticion[la_i].numReg = hmi_peticion_aux.numReg;
					mdbm_poll.peticion[la_i].pInst = hmi_peticion_aux.pInst;
					mdbm_poll.peticion[la_i].tiempoBase = 0;

					la_encontrado = 1;
				}
			}
			if (la_encontrado == 0)
			{
				hmi_mensajeNoEspacio ();
			}
		}
	}
	hmi_menu();
}

/*****************************************************************************
** Function name:		hmi_opcionConsultaDatos
*****************************************************************************/
static void hmi_opcionConsultaDatos(char *par_cadena)
{
	unsigned char la_car;

	la_car = *par_cadena;
	tipoPunto = 0;
	switch (la_car)
	{
		/* Muestra menu */
		case '0':
			hmi_menu();
		break;

		/* Coil status */
		case '1':
			tipoPunto = 1;
			/* peticion de numero de punto */
			hmi_puntoConsulta();
		break;

		/* Input status */
		case '2':
			tipoPunto = 2;
			/* peticion de numero de punto */
			hmi_puntoConsulta();
		break;

		/* Holding register */
		case '3':
			tipoPunto = 3;
			/* peticion de numero de punto */
			hmi_puntoConsulta();
		break;

		/* Input register */
		case '4':
			tipoPunto = 4;
			/* peticion de numero de punto */
			hmi_puntoConsulta();
		break;

		default:
			hmi_opcionNoValida();
    }
}

/*****************************************************************************
** Function name:		hmi_opcionPuntoMuestra
*****************************************************************************/
static void hmi_opcionPuntoMuestra(char *par_cadena)
{
	unsigned short la_punto;
	char la_aux[64];

	la_punto = atoi(par_cadena);

	switch (tipoPunto)
	{
		case 1:
			if (la_punto < RD_MAX_CS)
			{
				/* Consulta punto y mustra por pantalla */
				sprintf (la_aux,"\r\n\r\n Punto Coil Status: %d  Valor: %d", la_punto, reg_datosLeerCS(la_punto));
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
			}
			else
			{
				/* Indica que el punto esta fuera de rango */
				sprintf (la_aux,"\r\n\r\n Punto Maximo Coil Status: %d ", RD_MAX_CS);
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
			}
		break;

		case 2:
			if (la_punto < RD_MAX_IS)
			{
				/* Consulta punto y mustra por pantalla */
				sprintf (la_aux,"\r\n\r\n Punto Input Status: %d  Valor: %d", la_punto, reg_datosLeerIS(la_punto));
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
			}
			else
			{
				/* Indica que el punto esta fuera de rango */
				sprintf (la_aux,"\r\n\r\n Punto Maximo Input Status: %d ", RD_MAX_IS);
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
			}
		break;

		case 3:
			if (la_punto < RD_MAX_HR)
			{
				/* Consulta punto y mustra por pantalla */
				sprintf (la_aux,"\r\n\r\n Punto Holding Register: %d  Valor: %d", la_punto, reg_datosLeerHR(la_punto));
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
			}
			else
			{
				/* Indica que el punto esta fuera de rango */
				sprintf (la_aux,"\r\n\r\n Punto Maximo Holding Registers: %d ", RD_MAX_HR);
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
			}
		break;

		case 4:
			if (la_punto < RD_MAX_IR)
			{
				/* Consulta punto y mustra por pantalla */
				sprintf (la_aux,"\r\n\r\n Punto Input Register: %d  Valor: %d", la_punto, reg_datosLeerIR(la_punto));
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
			}
			else
			{
				/* Indica que el punto esta fuera de rango */
				sprintf (la_aux,"\r\n\r\n Punto Maximo Input Registers: %d ", RD_MAX_IR);
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
			}
		break;

		default:
		break;
	}

	hmi_menu();
}


/*****************************************************************************
** Function name:		hmi_opcionModificaDatos
*****************************************************************************/
static void hmi_opcionModificaDatos(char *par_cadena)
{
	unsigned char la_car;

	la_car = *par_cadena;
	tipoPunto = 0;
	switch (la_car)
	{
		/* Muestra menu */
		case '0':
			hmi_menu();
		break;

		/* Coil status */
		case '1':
			tipoPunto = 1;
			/* peticion de numero de punto */
			hmi_puntoModifica();
		break;

		/* Input status */
		case '2':
			tipoPunto = 2;
			/* peticion de numero de punto */
			hmi_puntoModifica();
		break;

		/* Holding register */
		case '3':
			tipoPunto = 3;
			/* peticion de numero de punto */
			hmi_puntoModifica();
		break;

		/* Input register */
		case '4':
			tipoPunto = 4;
			/* peticion de numero de punto */
			hmi_puntoModifica();
		break;

		default:
			hmi_opcionNoValida();
    }
}

/*****************************************************************************
** Function name:		hmi_opcionPuntoModifica
*****************************************************************************/
static void hmi_opcionPuntoModifica(char *par_cadena)
{
	char la_aux[64];
	uint8_t  la_salir = 0;

	numPunto = atoi(par_cadena);

	switch (tipoPunto)
	{
		case 1:
			if (numPunto >= RD_MAX_CS)
			{
				/* Indica que el punto esta fuera de rango */
				sprintf (la_aux,"\r\n\r\n Punto Maximo Coil Status: %d ", RD_MAX_CS);
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
				la_salir = 1;
			}
		break;

		case 2:
			if (numPunto >= RD_MAX_IS)
			{
				/* Indica que el punto esta fuera de rango */
				sprintf (la_aux,"\r\n\r\n Punto Maximo Input Status: %d ", RD_MAX_IS);
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
				la_salir = 1;
			}
		break;

		case 3:
			if (numPunto >= RD_MAX_HR)
			{
				/* Indica que el punto esta fuera de rango */
				sprintf (la_aux,"\r\n\r\n Punto Maximo Holding Registers: %d ", RD_MAX_HR);
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
				la_salir = 1;
			}
		break;

		case 4:
			if (numPunto >= RD_MAX_IR)
			{
				/* Indica que el punto esta fuera de rango */
				sprintf (la_aux,"\r\n\r\n Punto Maximo Input Registers: %d ", RD_MAX_IR);
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
				la_salir = 1;
			}
		break;

		default:
			la_salir = 1;
		break;
	}

	if (la_salir == 1)
	{
		hmi_menu();
	}
	else
	{
		hmi_menuPuntoValor();
	}
}

/*****************************************************************************
** Function name:		hmi_opcionPuntoValor
*****************************************************************************/
static void hmi_opcionPuntoValor(char *par_cadena)
{
	unsigned short la_valor;

	la_valor = atoi(par_cadena);

	switch (tipoPunto)
	{
		case 1:
			if (la_valor)
			{
				reg_datosEscribeCS (numPunto, 1);
			}
			else
			{
				reg_datosEscribeCS (numPunto, 0);
			}
		break;

		case 2:
			if (la_valor)
			{
				reg_datosEscribeIS (numPunto, 1);
			}
			else
			{
				reg_datosEscribeIS (numPunto, 0);
			}
		break;

		case 3:
			reg_datosEscribeHR (numPunto, la_valor);
		break;

		case 4:
			reg_datosEscribeIR (numPunto, la_valor);
		break;

		default:
		break;
	}

	hmi_menu();
}

/*****************************************************************************
** Function name:		hmi_opcionVelocidadS
*****************************************************************************/
static void hmi_opcionVelocidadS (char *par_cadena)
{
	unsigned char la_car;

	la_car = *par_cadena;
	switch (la_car)
	{
		/* Muestra menu */
		case '0':
			hmi_menu();
		break;

		/* Velocidad 9600 */
		case '1':
			UARTInit(UART_MDBE, 9600);
			hmi_menu();
		break;

		/* Velocidad 19200 */
		case '2':
			UARTInit(UART_MDBE, 19200);
			hmi_menu();
		break;

		/* Velocidad 38400 */
		case '3':
			UARTInit(UART_MDBE, 38400);
			hmi_menu();
		break;

		default:
			hmi_opcionNoValida();
    }
}

/*****************************************************************************
** Function name:		hmi_opcionSlaveAddress
*****************************************************************************/
static void hmi_opcionSlaveAddress (char *par_cadena)
{
	unsigned char la_address;

	la_address = atoi(par_cadena);

	if (la_address == 0)
	{
		hmi_menu();
	}
	else if ((la_address > 0) && (la_address < 248))
	{
		mdbe_poll.dispositivo = la_address;
		hmi_menu();
	}
	else
	{
		hmi_menuSlaveAddress();
	}
}
