/*
 * reg_datos.c
 *
 *  Created on: 12/05/2014
 *      Author: Alfredo Florencio Jimenez
 */

#include "FreeRTOS.h"
#include "LPC17xx.h"
#include "reg_datos.h"
#include "semphr.h"
#include "stdio.h"
#include "uart.h"
#include "string.h"

xSemaphoreHandle semaforoReg_datos = NULL;

extern unsigned char hmiEvent;
extern uint32_t UART_activa;

/*****************************************************************************
** Function name:		reg_datos_inicia
*****************************************************************************/
uint8_t reg_datos_inicia (void)
{
	uint16_t la_i; /* Indice de recorrido */

	/* Inicia a 0 los valores de los puntos para el mapeado de los Coil Status */
	for (la_i = 0; la_i < RD_MAX_CS; la_i++)
	{
		RD_CoilStatus[la_i] = 0;
	}

	/* Inicia a 0 los valores de los puntos para el mapeado de los Input Status */
	for (la_i = 0; la_i < RD_MAX_IS; la_i++)
	{
		RD_InputStatus[la_i] = 0;
	}

	/* Inicia a 0 los valores de los puntos para el mapeado de los Holding Registers */
	for (la_i = 0; la_i < RD_MAX_HR; la_i++)
	{
		RD_HoldingRegisters[la_i] = 0;
	}

	/* Inicia a 0 los valores de los puntos para el mapeado de los Input Registers */
	for (la_i = 0; la_i < RD_MAX_IR; la_i++)
	{
		RD_InputRegisters[la_i] = 0;
	}

	semaforoReg_datos = xSemaphoreCreateMutex();
	if (semaforoReg_datos == NULL)
	{
		return 0;
	}

	return 1;
}

/*****************************************************************************
** Function name:		reg_datosEscribeCS
*****************************************************************************/
void reg_datosEscribeCS (uint16_t ar_punto, uint8_t ar_valor)
{
	char la_aux[50];
	if (ar_punto < RD_MAX_CS)
	{
		xSemaphoreTake( semaforoReg_datos,  portMAX_DELAY );
		{
			RD_CoilStatus[ar_punto] = ar_valor;
			if (hmiEvent)
			{
				sprintf (la_aux,"\r\n CS - Punto: %d  Valor: %d", ar_punto, ar_valor);
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
			}
		}
		xSemaphoreGive( semaforoReg_datos );
	}
}

/*****************************************************************************
** Function name:		reg_datosEscribeIS
*****************************************************************************/
void reg_datosEscribeIS (uint16_t ar_punto, uint8_t ar_valor)
{
	char la_aux[50];
	if (ar_punto < RD_MAX_IS)
	{
		xSemaphoreTake( semaforoReg_datos,  portMAX_DELAY );
		{
			RD_InputStatus[ar_punto] = ar_valor;
			if (hmiEvent)
			{
				sprintf (la_aux,"\r\n IS - Punto: %d  Valor: %d", ar_punto, ar_valor);
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
			}
		}
		xSemaphoreGive( semaforoReg_datos );
	}
}

/*****************************************************************************
** Function name:		reg_datosEscribeHR
*****************************************************************************/
void reg_datosEscribeHR (uint16_t ar_punto, short ar_valor)
{
	char la_aux[50];
	if (ar_punto < RD_MAX_HR)
	{
		xSemaphoreTake( semaforoReg_datos,  portMAX_DELAY );
		{
			RD_HoldingRegisters[ar_punto] = ar_valor;
			if (hmiEvent)
			{
				sprintf (la_aux,"\r\n HR - Punto: %d  Valor: %d", ar_punto, ar_valor);
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
			}
		}
		xSemaphoreGive( semaforoReg_datos );
	}
}

/*****************************************************************************
** Function name:		reg_datosEscribeIR
*****************************************************************************/
void reg_datosEscribeIR (uint16_t ar_punto, short ar_valor)
{
	char la_aux[50];
	if (ar_punto < RD_MAX_IR)
	{
		xSemaphoreTake( semaforoReg_datos,  portMAX_DELAY );
		{
			RD_InputRegisters[ar_punto] = ar_valor;
			if (hmiEvent)
			{
				sprintf (la_aux,"\r\n IR - Punto: %d  Valor: %d", ar_punto, ar_valor);
				UARTSend (UART_activa, (uint8_t *)la_aux, strlen(la_aux));
			}
		}
		xSemaphoreGive( semaforoReg_datos );
	}
}

/*****************************************************************************
** Function name:		reg_datosLeerCS
*****************************************************************************/
uint8_t reg_datosLeerCS (uint16_t ar_punto)
{
	uint8_t la_valor;

	if (ar_punto < RD_MAX_CS)
	{
		xSemaphoreTake( semaforoReg_datos,  portMAX_DELAY );
		{
			la_valor = RD_CoilStatus[ar_punto];
		}
		xSemaphoreGive( semaforoReg_datos );
	}
	return la_valor;
}

/*****************************************************************************
** Function name:		reg_datosLeerIS
*****************************************************************************/
uint8_t reg_datosLeerIS (uint16_t ar_punto)
{
	uint8_t la_valor;

	if (ar_punto < RD_MAX_IS)
	{
		xSemaphoreTake( semaforoReg_datos,  portMAX_DELAY );
		{
			la_valor = RD_InputStatus[ar_punto];
		}
		xSemaphoreGive( semaforoReg_datos );
	}
	return la_valor;
}

/*****************************************************************************
** Function name:		reg_datosLeerHR
*****************************************************************************/
short reg_datosLeerHR (uint16_t ar_punto)
{
	short la_valor;

	if (ar_punto < RD_MAX_HR)
	{
		xSemaphoreTake( semaforoReg_datos,  portMAX_DELAY );
		{
			la_valor = RD_HoldingRegisters[ar_punto];
		}
		xSemaphoreGive( semaforoReg_datos );
	}
	return la_valor;
}

/*****************************************************************************
** Function name:		reg_datosLeerIR
*****************************************************************************/
short reg_datosLeerIR (uint16_t ar_punto)
{
	short la_valor;

	if (ar_punto < RD_MAX_IR)
	{
		xSemaphoreTake( semaforoReg_datos,  portMAX_DELAY );
		{
			la_valor = RD_InputRegisters[ar_punto];
		}
		xSemaphoreGive( semaforoReg_datos );
	}
	return la_valor;
}
