/*
 * wifly.c
 *
 *  Created on: 01/04/2014
 *      Author: Usuario
 */
#include "FreeRTOS.h"
#include "LPC17xx.h"
#include "wifly.h"
#include "string.h"
#include "task.h"
#include "uart.h"
#include "stdlib.h"
#include "printUART.h"
#include "semphr.h"
#include "stdio.h"
#include "queue.h"
#include "utils.h"

char TX_BUFF[BUF_LONG];
char RX_BUFF[BUF_LONG];
char RX_BAUX[BUF_LONG];
char STR_VAL[BUF_LONG];

uint16_t sizeAsk;
uint16_t aux;

extern volatile uint32_t UART0Count;
extern volatile uint8_t UART0Buffer[BUFSIZE];

xSemaphoreHandle semWiFly = NULL;

/* Declaracion de funciones privadas del módulo */

uint8_t wifly_SendCMD (char *cmd, uint32_t timeout);
uint8_t wifly_SendCMDAOK (char *cmd);
uint8_t wifly_Open (void);
uint16_t wifly_ConfigDNS (char *dns);
uint8_t wifly_ModoComando (void);
char* wifly_parse(char *subC, char *fin, char *text, uint8_t tam);


/*****************************************************************************
** Function name:		wifly_SendCMD
*****************************************************************************/

uint8_t wifly_SendCMD (char *cmd, uint32_t timeout)
{
	memset(RX_BUFF, 0x00, BUF_LONG);
	//printUART_Text(UART_WIFLY, cmd);
	//UARTSend (UART_WIFLY, (uint8_t *)HMI_M_CA, strlen(HMI_M_CA));
	UARTSend (UART_WIFLY, (uint8_t *)cmd, strlen(cmd));
	return (UART0Scan (RX_BUFF, timeout));
}

/*****************************************************************************
** Function name:		wifly_SendCMDAOK
*****************************************************************************/
uint8_t wifly_SendCMDAOK (char *cmd)
{
	uint8_t ret = 0;

	/* Enviar el mensaje */
	sizeAsk = wifly_SendCMD (cmd, TM_ACK);
	//printUART_Text(UART_LOG, RX_BUFF);

	if (wifly_parse("AO", "K", RX_BUFF, sizeAsk) != NULL)
	{
		ret = 1;
	}

	return ret;
}

/*****************************************************************************
** Function name:		wifly_Open
*****************************************************************************/
uint8_t wifly_Open (void)
{
	uint8_t ret = 0;

	/* Enviar el mensaje */
	sizeAsk = wifly_SendCMD (WF_OPEN, TM_OPEN);

	/* log para respuesta al comando */
	//printUART_Text(UART_LOG, RX_BUFF);

	if (wifly_parse("OPEN", "*", RX_BUFF, sizeAsk) != NULL)
	{
		ret = 1;
	}
	return (ret);
}


/*****************************************************************************
** Function name:		wifly_Ping
*****************************************************************************/
uint8_t wifly_Ping (char *address)
{
	uint8_t ret = 0;

	xSemaphoreTake( semWiFly,  portMAX_DELAY );
	{
		sizeAsk = 0;
		memset(TX_BUFF, 0x00, BUF_LONG);
		strcpy (TX_BUFF, "ping ");
		strcat (TX_BUFF, address);
		strcat (TX_BUFF,"\r\n");

		/* Enviar el mensaje */
		sizeAsk = wifly_SendCMD (TX_BUFF, TM_PING);

		/* Log para comprobar la respuesta del comando */
		//printUART_Text(UART_LOG, RX_BUFF);

		if (wifly_parse("time=", "m", RX_BUFF, sizeAsk) != NULL)
		{
			ret = 1;
		}
	}
	xSemaphoreGive( semWiFly );
	return (ret);
}


/*****************************************************************************
** Function name:		wifly_IP
*****************************************************************************/
char* wifly_IP (void)
{
	char *ret = STR_VAL;

	xSemaphoreTake( semWiFly,  portMAX_DELAY );
	{
		sizeAsk = 0;

		/* Enviar el mensaje */
		sizeAsk = wifly_SendCMD (WF_GET_IP, TM_ACK);

		/* Log para la respuesta del WiFly */
		//printUART_Text(UART_LOG, RX_BUFF);

		ret = wifly_parse("IP=", ":", RX_BUFF, sizeAsk);
	}
	xSemaphoreGive( semWiFly );
	return (ret);
}


/*****************************************************************************
** Function name:		wifly_Canal
*****************************************************************************/
char* wifly_Canal (void)
{
	char *ret = STR_VAL;

	xSemaphoreTake( semWiFly,  portMAX_DELAY );
	{
		sizeAsk = 0;

		/* Enviar el mensaje */
		sizeAsk = wifly_SendCMD (WF_GET_CH, TM_ACK);

		/* log para respuesta al comando */
		//printUART_Text(UART_LOG, RX_BUFF);

		ret = wifly_parse("Chan=", "\r", RX_BUFF, sizeAsk);
	}
	xSemaphoreGive( semWiFly );
	return (ret);
}


/*****************************************************************************
** Function name:		wifly_RSSI
*****************************************************************************/
char* wifly_RSSI (void)
{
	char *ret = STR_VAL;

	xSemaphoreTake( semWiFly,  portMAX_DELAY );
	{
		sizeAsk = 0;

		/* Enviar el mensaje */
		sizeAsk = wifly_SendCMD (WF_GET_RSSSI, TM_ACK);

		/* log para comprobar salida del comando */
		//printUART_Text(UART_LOG, RX_BUFF);

		ret = wifly_parse("RSSI=(", ")", RX_BUFF, sizeAsk);
	}
	xSemaphoreGive( semWiFly );
	return (ret);
}


/*****************************************************************************
** Function name:		wifly_Tiempo
*****************************************************************************/
char* wifly_Tiempo (void)
{
	char *ret = STR_VAL;

	xSemaphoreTake( semWiFly,  portMAX_DELAY );
	{
		sizeAsk = 0;

		/* Enviar el mensaje */
		sizeAsk = wifly_SendCMD (WF_GET_TIME, TM_ACK);

		/* log para comprobar salida del comando */
		//printUART_Text(UART_LOG, RX_BUFF);

		ret = wifly_parse("UpTime=", " ", RX_BUFF, sizeAsk);
	}
	xSemaphoreGive( semWiFly );
	return (ret);
}


/*****************************************************************************
** Function name:		wifly_ConsultaPotencia
*****************************************************************************/
char* wifly_ConsultaPotencia (void)
{
	char *ret = STR_VAL;

	xSemaphoreTake( semWiFly,  portMAX_DELAY );
	{
		sizeAsk = 0;

		/* Enviar el mensaje */
		sizeAsk = wifly_SendCMD (WF_GET_PWD, TM_ACK);

		/* log para comprobar salida del comando */
		//printUART_Text(UART_LOG, RX_BUFF);

		ret = wifly_parse("TxPower=", "\r", RX_BUFF, sizeAsk);
	}
	xSemaphoreGive( semWiFly );
	return (ret);
}


/*****************************************************************************
** Function name:		wifly_CambiaPotencia
*****************************************************************************/
uint8_t wifly_CambiaPotencia (uint8_t potencia)
{
	char n[4];
	uint8_t ret = 0;

	xSemaphoreTake( semWiFly,  portMAX_DELAY );
	{
		memset(TX_BUFF, 0x00, BUF_LONG);

		strcat (TX_BUFF,"set wlan tx ");
		strcat (TX_BUFF, itoa(potencia,n,10));
		strcat (TX_BUFF,"\r\n");

		/* Enviar el mensaje */
		sizeAsk = wifly_SendCMD (TX_BUFF, TM_ACK);

		//printUART_Text(UART_LOG, RX_BUFF);

		if (wifly_parse("AO", "K", RX_BUFF, sizeAsk) != NULL)
		{
			ret = 1;
		}
	}
	xSemaphoreGive( semWiFly );
	return(ret);
}


/*****************************************************************************
** Function name:		wifly_EnviaGET
*****************************************************************************/
char* wifly_EnviaGET (char *command, char *host)
{

	xSemaphoreTake( semWiFly,  portMAX_DELAY );
	{
		/* forma el comando GET */
		memset(TX_BUFF, 0x00, BUF_LONG);

		strcat (TX_BUFF,"set com remote GET$");
		strcat (TX_BUFF, command);
		strcat (TX_BUFF,"\r\n");

		wifly_SendCMDAOK(TX_BUFF);

		wifly_ConfigDNS (host);

		memset(RX_BUFF, 0x00, BUF_LONG);
		//wifly_Open ();
		if (wifly_Open() == 0)
		{
			xSemaphoreGive( semWiFly );
			return NULL;
		}

		sizeAsk = UART0Scan (RX_BAUX, 20000);
		//printUART_Text(UART_LOG, RX_BAUX);

		wifly_ModoComando();
	}
	xSemaphoreGive( semWiFly );

	return RX_BAUX;
}


/*****************************************************************************
** Function name:		wifly_ConfigDNS
*****************************************************************************/
uint16_t wifly_ConfigDNS  (char *dns)
{
	uint16_t ret = 0;

	memset(TX_BUFF, 0x00, BUF_LONG);

	/* AFJ: prueba, cambio de name a address */
	strcat (TX_BUFF,"set dns name ");
	strcat (TX_BUFF, dns);
	strcat (TX_BUFF,"\r\n");

	/* Enviar el mensaje */
	sizeAsk = wifly_SendCMD (TX_BUFF, TM_ACK);

	//printUART_Text(UART_LOG, RX_BUFF);

	if (wifly_parse("AO", "K", RX_BUFF, sizeAsk) != NULL)
	{
		ret = 1;
	}
	return(ret);
}


/*****************************************************************************
** Function name:		wifly_Reset
*****************************************************************************/

uint8_t wifly_Reset(void)
{
	uint8_t la_ret = 1;

	xSemaphoreTake( semWiFly,  portMAX_DELAY );
	{
		/* Esperamos por si no estÃ¡ listo */
		memset(RX_BUFF, 0x00,BUF_LONG);

		delayMS (5000);

		/* Initialise P0_22 for the LED. */
		LPC_PINCON->PINSEL1	&= ( ~( 3 << 12 ) );
		LPC_GPIO0->FIODIR |= ( 1 << RST_WIFLY_PIN );

		/* Realizamos el reset del WiFly */
		LPC_GPIO0->FIOCLR = 1 << RST_WIFLY_PIN;
		delayMS (1);
		LPC_GPIO0->FIOSET = 1 << RST_WIFLY_PIN;

		/* Esperamos el reinicio del WiFly */
		delayMS (4000);

		/* Recogemos la respuesta para analizar */
		sizeAsk = UART0Scan (RX_BUFF, TM_RESET);

		/* Linea de test para comprobar salida del reset */
		//printUART_Text (UART_LOG, RX_BUFF);

		if (wifly_parse("Listen on ", "\r", RX_BUFF, sizeAsk) == NULL)
		{
			la_ret  = 0;
		}

		if (la_ret)
		{
			wifly_ModoComando();
		}
	}
	xSemaphoreGive( semWiFly );

	return la_ret;
}


/*****************************************************************************
** Function name:		Wifly_ModoComando
*****************************************************************************/

uint8_t wifly_ModoComando (void)
{
	uint8_t ret = 0;

	/* Enviar el mensaje */
	sizeAsk = wifly_SendCMD (WF_CMD_MODE, TM_ACK);

	/* Log para comprobar la respuesta del comando */
	//printUART_Text(UART_LOG, RX_BUFF);

	if (wifly_parse("CM", "D", RX_BUFF, sizeAsk) != NULL)
	{
		ret = 1;
	}
	return(ret);
}


/*****************************************************************************
** Function name:		wifly_ConectaWEP
*****************************************************************************/

uint8_t wifly_ConectaWEP (char *key)
{
	uint8_t ret = 0;

	xSemaphoreTake( semWiFly,  portMAX_DELAY );
	{
		memset(TX_BUFF, 0x00, BUF_LONG);

		strcat (TX_BUFF,"set wlan key ");
		strcat (TX_BUFF, key);
		strcat (TX_BUFF,"\r\n");

		/* Enviar el mensaje */
		sizeAsk = wifly_SendCMD (TX_BUFF, TM_ACK);

		//printUART_Text(UART_LOG, RX_BUFF);

		if (wifly_parse("AO", "K", RX_BUFF, sizeAsk) != NULL)
		{
			ret = 1;
		}
	}
	xSemaphoreGive( semWiFly );
	return(ret);
}


/*****************************************************************************
** Function name:		wifly_ConectaWAP
*****************************************************************************/

uint8_t wifly_ConectaWAP (char *phrase)
{
	uint8_t ret = 0;

	xSemaphoreTake( semWiFly,  portMAX_DELAY );
	{
		memset(TX_BUFF, 0x00, BUF_LONG);

		strcat (TX_BUFF,"set wlan phrase ");
		strcat (TX_BUFF, phrase);
		strcat (TX_BUFF,"\r\n");

		/* Enviar el mensaje */
		sizeAsk = wifly_SendCMD (TX_BUFF, TM_ACK);

		//printUART_Text(UART_LOG, RX_BUFF);

		if (wifly_parse("AO", "K", RX_BUFF, sizeAsk) != NULL)
		{
			ret = 1;
		}
	}
	xSemaphoreGive( semWiFly );
	return(ret);
}


/*****************************************************************************
** Function name:		wifly_ConectaSSID
*****************************************************************************/

uint8_t wifly_ConectaSSID (char *ssid)
{
	uint8_t ret = 0;

	xSemaphoreTake( semWiFly,  portMAX_DELAY );
	{
		memset(TX_BUFF, 0x00, BUF_LONG);

		strcat (TX_BUFF,"join ");
		strcat (TX_BUFF, ssid);
		strcat (TX_BUFF,"\r\n");

		/* Enviar el mensaje */
		sizeAsk = wifly_SendCMD (TX_BUFF, TM_JOIN);

		//printUART_Text(UART_LOG, RX_BUFF);

		if (wifly_parse("Associate", "d", RX_BUFF, sizeAsk) != NULL)
		{
			ret = 1;
		}
	}
	xSemaphoreGive( semWiFly );
	return(ret);
}


/*****************************************************************************
** Function name:		wifly_configHTTP
*****************************************************************************/
void wifly_configHTTP ()
{
//	xSemaphoreTake( semWiFly,  portMAX_DELAY );
//	{
		wifly_SendCMDAOK (WF_SET_PROTO);
		wifly_SendCMDAOK (WF_SET_HOST);
		wifly_SendCMDAOK (WF_SET_HTTP);
		wifly_SendCMDAOK (WF_SET_COMR);
		wifly_SendCMDAOK (WF_SET_OF_1);
//	}
//	xSemaphoreGive( semWiFly );

}

/*****************************************************************************
** Function name:		wifly_Init
*****************************************************************************/

uint8_t wifly_Init (void)
{
	/* Iniciamos la UART del modulo wifly */
	UARTInit(UART_WIFLY, BAUD_UART);

	/* Creamos el semaforo para la UART del WiFly */
	semWiFly = xSemaphoreCreateMutex();

	/* Comprobar la creaciÃ³n del semaforo */
	if (semWiFly == NULL)
	{
		return 0;
	}

	return 1;
}



/*****************************************************************************
** Function name:		wifly_parse
*****************************************************************************/

char* wifly_parse(char *subC, char *fin, char *text, uint8_t tam)
{
	uint8_t encontrado = 0;
	uint8_t i=0, j=0, k=0, l=0;

	l = strlen(subC);
	memset(STR_VAL, 0x00, BUF_LONG);

	for (i=0; i < tam; i++)
	{
		if (encontrado)
		{
			if (text[i] != fin[0])
			{
				STR_VAL[k] = text[i];
				k++;
			}
			else
			{
				STR_VAL[k] = 0;
				i = tam;
			}
		}

		if (subC[j] == text[i])
		{
			j++;
			if (j == l)
			{
				encontrado = 1;
			}
		}
		else
		{
			j=0;
		}

	}
	if (encontrado)
	{
		/* Log para el parse encontrado **/
		//printUART_Text(UART_LOG, STR_VAL);
		return STR_VAL;
	}
	/* Este else funciona como log para detectar cunado no se encuentra una cadena */
	else
	{
		//printUART_Text(UART_LOG, "\r\ncadena no encontrada\r\n");
	}

	return (NULL);
}

/*****************************************************************************
** Function name:		wifly_configTelnet
*****************************************************************************/
uint8_t wifly_configTelnet ()
{
	uint8_t la_ret = 0;

	wifly_SendCMDAOK ("\r\n");
	/* Configura IP */
	la_ret = wifly_SendCMDAOK ("set ip a 192.168.0.12\r\n");

	/* Configuracion del tipo de Wifi */
	if (la_ret)
	{
		la_ret = wifly_SendCMDAOK ("set wlan auth 8\r\n");
	}

	/* Indicacion de la red */
	if (la_ret)
	{
		la_ret = wifly_SendCMDAOK ("set wlan ssid WebSTAR\r\n");
	}

	/* Password de acceso */
	if (la_ret)
	{
		la_ret = wifly_SendCMDAOK ("set wlan key 6551c53530\r\n");
	}

	/* Modo de uart */
	if (la_ret)
	{
		la_ret = wifly_SendCMDAOK ("set uart mode 2\r\n");
	}

	/* Modo de uart */
	if (la_ret)
	{
		wifly_SendCMDAOK ("join\r\n");
	}

	return la_ret;
}

/******************************************************************************
**                            End Of File
******************************************************************************/


